package il.ac.tau.cs.smlab.model2lejos.library.wrappers.differentialpilot;
import il.ac.tau.cs.smlab.model2lejos.library.wrappers.differentialpilot.types.MotorCmd;
import il.ac.tau.cs.smlab.model2lejos.library.wrappers.differentialpilot.types.MotorSpeed;
import lejos.nxt.Motor;
import lejos.nxt.MotorPort;
import lejos.robotics.RegulatedMotor;

/**
 * Implementation of the component DifferentialPilot, used to move a two-wheeled robot
 * 
 * Attention 1: 
 * This class needs to be refactored and improved, because it is completely specified to the constructed robot
 * - while turning the the robot the color sensors have to turn on the spot, so before turning around 90°
 *     the robot needs to go straight for about 13cm
 * 
 * Attention 2:
 * This class containes cycle blocking commands, which could lead to some strange behavior!
 */
public class DifferentialPilotImpl extends DifferentialPilot {

	/** instance of lejos DifferentialPilot */
	private lejos.robotics.navigation.DifferentialPilot pilot;
	/** distance for a travel command*/
	private double dDistance;
	
	/**
	 * Constructor, to initialize the pilot
	 * @param lWheelDiameter
	 * 			diameter of the left tire in inch
	 * @param rWheelDiameter
	 * 			diameter of the left tire in inch
	 * @param left
	 * 			the left Motor 
	 * @param right
	 * 			the right Motor 
	 */
	public DifferentialPilotImpl(double lWheelDiameter, double rWheelDiameter, 
							lejos.nxt.MotorPort left, lejos.nxt.MotorPort right) 
	{
	  super(lWheelDiameter, rWheelDiameter, left, right);
	  dDistance = 3.5;
	}

	/**
	 * Set up the DifferentialPilot and set slow speed
	 */
	@Override
    public void setUp() {
		  super.setUp();
		  if(this.right != null && this.left != null ) {
			  RegulatedMotor LeftMotor=Motor.A;
			  RegulatedMotor RightMotor=Motor.A;
			  if (this.left==lejos.nxt.MotorPort.B) {
				  LeftMotor=Motor.B;
			  } else if (this.left==lejos.nxt.MotorPort.C) {
				  LeftMotor=Motor.C;
			  }
			  if (this.right==lejos.nxt.MotorPort.B) {
				  RightMotor=Motor.B;
			  } else if (this.right==lejos.nxt.MotorPort.C) {
				  RightMotor=Motor.C;
			  }
			  
			  pilot = new lejos.robotics.navigation.DifferentialPilot(this.lWheelDiameter, this.rWheelDiameter, 
					  LeftMotor, RightMotor, false);
		  
		  // slow inital speeds
		  pilot.setTravelSpeed(2);
		  pilot.setRotateSpeed(90);
		  }
    }

	/**
	 * React on commands set to the inports of the DifferentialPilot component
	 * The order is first travel command with distance, the 90° left, 90° right, 180°, 
	 * then MotorCmds in the order of the enum MotorCmd
	 */
    @Override
    public void compute() 
    {	  
    	
    	  if(this.travelSpeed != null && this.travelSpeed.getCurrentValue() != null)
		  {
    		  pilot.setTravelSpeed(this.travelSpeed.getCurrentValue());
		  }
    	  
    	  
    	  if(this.discrete_speed != null && this.discrete_speed.getCurrentValue() != null)
		  {
				
			  double speed_unit=pilot.getMaxTravelSpeed()/MotorSpeed.values().length;
			  
			  switch(this.discrete_speed.getCurrentValue()) {
			  	
			  	case SPEED1: {
			  		pilot.setTravelSpeed(1*speed_unit);
			  		break;
			  		}
			  	case SPEED2: {
			  		pilot.setTravelSpeed(2*speed_unit);
			  		break;
			  		}
			  	case SPEED3: {
			  		pilot.setTravelSpeed(3*speed_unit);
			  		break;
			  		}
			  	case SPEED4: {
			  		pilot.setTravelSpeed(4*speed_unit);
			  		break;
			  		}
			  	case SPEED5: {
			  		pilot.setTravelSpeed(5*speed_unit);
			  		break;
			  		}
			  	case SPEED6: {
			  		pilot.setTravelSpeed(6*speed_unit);
			  		break;
			  		}
			  	case SPEED7: {
			  		pilot.setTravelSpeed(7*speed_unit);
			  		break;
			  		}
			  	case SPEED8: {
			  		pilot.setTravelSpeed(8*speed_unit);
			  		break;
			  		}
			  	case SPEED9: {
			  		pilot.setTravelSpeed(9*speed_unit);
			  		break;
			  		}
			  	case SPEED10: {
			  		pilot.setTravelSpeed(10*speed_unit);
			  		break;
			  		}
			  	
	  	
			  	
			  }
		  }
    	  
    	  
    	  
    	  
    	  if(this.rotateSpeed != null && this.rotateSpeed.getCurrentValue() != null)
		  {
    		  pilot.setTravelSpeed(this.rotateSpeed.getCurrentValue());
		  }
		  if(this.distance != null && this.distance.getCurrentValue() != null)
		  {
			  // travel a distance of x inch
			  dDistance = this.distance.getCurrentValue().doubleValue() / 2.54;
			//  System.out.println(dDistance);
			 pilot.travel(dDistance);
			 pilot.stop();
			 pilot.reset();
		  }
		   
		  if(this.bTurn90Left != null && this.bTurn90Left.getCurrentValue() != null && this.bTurn90Left.getCurrentValue())
		  {
			  // travel a distance of 5.1 inch and then rotate around -90°, with the color sensors on the spot
			  pilot.travel(5.1); // about 13 cm
			  pilot.rotate(-310); // turn vehicle around 90°
			  pilot.stop();
			  pilot.reset();
		  }
		  
		  if(this.bTurn90Right != null && this.bTurn90Right.getCurrentValue() != null && this.bTurn90Right.getCurrentValue())
		  {
			  // travel a distance of 5.1 inch and then rotate around -90°, with the color sensors on the spot
			  pilot.travel(5.1); // about 13 cm
			  pilot.rotate(310);  // turn vehicle around 90°
			  pilot.stop();
			  pilot.reset();
		  }
		  
		  if(this.bTurn180 != null && this.bTurn180.getCurrentValue() != null && this.bTurn180.getCurrentValue())
		  {
			  
			  pilot.rotate(620);  // turn vehicle around 180°
			  pilot.stop();
			  pilot.reset();
		  }
		  
		  			  
			  
			  
			  
			  
			  if(this.cmd != null && this.cmd.getCurrentValue() != null)
			  {
					
				  switch(this.cmd.getCurrentValue()) {
				  
				  	case FORWARD: {
				  		pilot.forward();
				  		break;
				  		}

				  	case STOP: {
				  		pilot.stop();
				  		break;
				  		}

				  	
				  	case ROTATELEFTINPLACE: {
				  		pilot.rotateLeft();
				  		break;
				  		}

				  	case ROTATERIGHTINPLACE: {
				  		pilot.rotateRight();
				  		break;
				  		}

				  	case STEERLEFT: {
				  		pilot.steer(30);
				  		break;
				  		}
				  
				  	case STEERRIGHT: {
				  		pilot.steer(-30);
				  		break;
				  		}
				  	
				  	
				  }
			  }

		}
  	}
